package com.isomorphic.examples.atmosphere.client;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.google.gwt.user.client.Timer;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.i18n.client.NumberFormat;

import com.isomorphic.atmosphere.client.Atmosphere;
import com.isomorphic.atmosphere.client.Configurator;

import com.smartgwt.client.rpc.RPCManager;
import com.smartgwt.client.rpc.RPCRequest;
import com.smartgwt.client.types.DSOperationType;
import com.smartgwt.client.util.JSOHelper;
import com.smartgwt.client.util.JSON;
import com.smartgwt.client.widgets.Button;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.grid.CellFormatter;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridRecord;
import com.smartgwt.client.widgets.layout.VLayout;
import com.smartgwt.client.data.DSRequest;
import com.smartgwt.client.data.DSResponse;

public class AtmosphereSample implements EntryPoint {
	
	private static ListGrid stockQuotesGrid;
	private static Logger logger = Logger.getLogger("MyLogger");
	
	public void onModuleLoad() {
		logger.log(Level.SEVERE, "Start AtmosphereSAmple application.");

		Canvas topCanvas=null;
		final long startParameter = System.currentTimeMillis();
		
		// Grid and button below it
		VLayout contentLayout = new VLayout();
		contentLayout.setMembersMargin(10);
		contentLayout.setPadding(10);
		
		stockQuotesGrid = new ListGrid() {
			private int blinkPeriod = 2000;

			// Cell animation - go bright green or red on a change, then fades.
			protected String getCellCSSText(ListGridRecord record, final int rowNum,
					final int colNum) {
				// changeValue column
				Date lastUpdated = record.getAttributeAsDate("lastUpdated");
				if (colNum == 2 && lastUpdated != null) {
					long delta = System.currentTimeMillis() - lastUpdated.getTime();
					if (delta < blinkPeriod) {
						// refresh 10x / second
						new Timer() {
							public void run() {
								stockQuotesGrid.refreshCell(rowNum, colNum);
							}
						}.schedule(100);
						float changeValue = record.getAttributeAsFloat("changeValue");

						float ratio = ((float) (blinkPeriod - delta)) / blinkPeriod;
						int color = 255 - Math.round(200 * ratio);

						if (changeValue > 0) {
							return "background-color:#" + Integer.toHexString(color) + "FF"
							+ Integer.toHexString(color);
						} else if (changeValue < 0) {
							return "background-color:#FF" + Integer.toHexString(color)
							+ Integer.toHexString(color);
						}
					} else {
						record.setAttribute("lastUpdated", (Date) null);
					}
				}
				// no style override
				return null;
			};
		};
		stockQuotesGrid.setWidth(600);
		stockQuotesGrid.setHeight(300);
		stockQuotesGrid.setShowAllRecords(true);
		stockQuotesGrid.setDataSource(StockQuotesDS.getInstance());
		stockQuotesGrid.setAutoFetchData(true);
		stockQuotesGrid.setCellFormatter(getFloatValuesFormatter());
		contentLayout.addMember(stockQuotesGrid);

		// Initialization of the atmosphere services.
		// The Configurator object contains all the data needed to
		// start the communications channel an to register the callback
		// method that will eb invoked when data is pushed from server.
		final String channelId = "" + startParameter; 
		Configurator configurator = new Configurator();
		configurator.setService("jersey/rpc");
		configurator.setChannelId(channelId);
		configurator.setTransport(Configurator.TRANSPORT_WEBSOCKET);
		configurator.setFallbackTransport(Configurator.TRANSPORT_LONG_POLLING);
		configurator.setMessageHandler(new AtmosphereMessageHandler());

		// Start listening remote events
		@SuppressWarnings("unused")
		final Atmosphere.AtmosphereSocket socket = Atmosphere.subscribe(configurator);
		
		final Button generateUpdatesButton = new Button("Generate more updates");
		generateUpdatesButton.setWidth(200);
		generateUpdatesButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				generateUpdates(startParameter, generateUpdatesButton);
			}
		});
		contentLayout.addMember(generateUpdatesButton);

		generateUpdates(startParameter, generateUpdatesButton);

		topCanvas = contentLayout;
		topCanvas.draw();
	}

    /**
     * Float values should be formatted into human-readable text
     * 
     * @return
     */
    private CellFormatter getFloatValuesFormatter() {
        return new CellFormatter() {
            public String format(Object value, ListGridRecord record, int rowNum, int colNum) {
                if (value == null)
                    return null;

                if (value instanceof Number) {
                    NumberFormat nf = NumberFormat.getFormat("0.00");
                    try {
                        return nf.format(((Number) value).floatValue());
                    } catch (Exception e) {
                        return value.toString();
                    }
                }
                return String.valueOf(value);
            }
        };
    }
    
	@SuppressWarnings("unchecked")
	public static void updateStockQuotes(String responseBody) {
		if(stockQuotesGrid==null) {
			logger.log(Level.INFO, "Message discarded, we are not ready!");
			return;
		}

		JavaScriptObject jsoResponseBody = JSON.decode(responseBody);
		Map<?,?> rp = (Map<?,?>) JSOHelper.convertToJava(jsoResponseBody);
		List<List<?>> stockData = (List<List<?>>) rp.get("stockData");
		
		List<ListGridRecord> newStockData = new ArrayList<ListGridRecord>();
    	
    	// prepare data for grid manually using received data from servlet  
        // we receive only 'id' and 'change percent' data here  
        for (List<?> recordData : stockData) {
            float change = ((Number) recordData.get(1)).floatValue();  
            if (change != 0) {
                Integer id = (Integer) recordData.get(0);  
                ListGridRecord record =
                		new ListGridRecord(stockQuotesGrid.getDataAsRecordList().find("id", id));  
                float lastValue = record.getAttributeAsFloat("lastValue");  
                float newChangeValue = change * lastValue / 100;  
                float newLastValue = newChangeValue + lastValue;  
                record.setAttribute("changeValue", newChangeValue);  
                record.setAttribute("lastValue", newLastValue);  
                record.setAttribute("dayHighValue",  
                        Math.max(record.getAttributeAsFloat("dayHighValue"), newLastValue));  
                record.setAttribute("dayLowValue",  
                        Math.min(record.getAttributeAsFloat("dayLowValue"), newLastValue));  
                record.setAttribute("lastUpdated", new Date());  
                newStockData.add(record);  
            }  
        }
        
    	DSResponse dsResponse = new DSResponse();
    	dsResponse.setData(newStockData.toArray(new ListGridRecord[newStockData.size()]));

    	DSRequest dsRequest = new DSRequest();
    	dsRequest.setOperationType(DSOperationType.UPDATE);

    	// broadcast the change - the grid will notice this automatically (and so would other
    	// components showing the same record)
    	StockQuotesDS.getInstance().updateCaches(dsResponse, dsRequest);
    }   
    
    /*
     * This method instructs the server to start sending back update messages, two per second
     */
    private void generateUpdates(final long startParameter, final Button generateUpdatesButton) {
    	generateUpdatesButton.disable();

    	RPCRequest request = new RPCRequest();
		// we tell servlet which channel it should use for sending data
    	request.setActionURL("examples/StockQuotes/generate?sp=" + startParameter);
    	RPCManager.sendRequest(request);

    	// block button repeat click for 90 seconds - time while servlet
    	// will sends data to us
    	new Timer() {
    		public void run() {
    			generateUpdatesButton.enable();
    		}
    	}.schedule(90000);
    }     
}