package com.isomorphic.examples.atmosphere.server;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;  
import java.util.Random;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;  
import javax.servlet.http.HttpServlet;  
import javax.servlet.http.HttpServletRequest;  
import javax.servlet.http.HttpServletResponse;  

import org.atmosphere.cpr.Broadcaster;
import org.atmosphere.cpr.BroadcasterFactory;
import org.atmosphere.util.ServletContextFactory;

/** 
 * Servlet for generating random data for stock quotes grid. 
 */  
public class StockQuotesServlet extends HttpServlet {  
	
	private static final long serialVersionUID = 6200724895227971697L;  

	public void doPost(HttpServletRequest request, HttpServletResponse response)  
	throws ServletException, IOException {  
		processRequest(request, response);  
	}  

	// RPC transactions may be encoded as a GET request, so handle those as well  
	public void doGet(HttpServletRequest request, HttpServletResponse response)  
	throws ServletException, IOException {  
		processRequest(request, response);  
	}  

	/** 
	 * Servlet entry point to process the request. sends random stock updates to 
	 * the "stockQuotes" channel for 30 seconds 
	 *  
	 * @param request The HttpServletRequest 
	 * @param response The HttpServletResponse 
	 * @exception ServletException As per HttpServlet.service() 
	 * @exception IOException As per HttpServlet.service() 
	 *  
	 * @visibility external 
	 */  
	public void processRequest(HttpServletRequest request,  
			HttpServletResponse response) throws ServletException, IOException {  
		final String channelId = request.getParameter("sp");  
		new Thread() {  
			public void run() {  
				try {  
					Random generator = new Random();
					
					ServletContext servletContext = ServletContextFactory.getDefault().getServletContext();
					BroadcasterFactory bf = (BroadcasterFactory) servletContext.getAttribute("org.atmosphere.cpr.BroadcasterFactory");
					Broadcaster channel = bf.lookup(channelId);

					List<Object[]> stockData = new ArrayList<Object[]>();
					long endTime = System.currentTimeMillis() + 90000;  
					while (System.currentTimeMillis() < endTime) {  
						stockData.clear();  
						for (int i = 1; i <= 10; i++) {  
							stockData.add(new Object[] { i, 0 });  
						}  
						int cnt = 1 + generator.nextInt(3);  
						for (int i = 0; i < cnt; i++) {  
							int id = 1 + generator.nextInt(10);  
							for (Object[] sd : stockData) {  
								if (((Integer) sd[0]).intValue() == id) {  
									sd[1] = ((float) (generator.nextInt(101) - 50)) / 100;  
									break;  
								}  
							}  
						}  

						JsonResponse response = new JsonResponse(stockData);
						channel.broadcast(response);
						
					    Thread.sleep(500);  
					}  
				} catch (Exception e) {  
					e.printStackTrace(System.out);  
				}  
			};  
		}.start();  
	}
}  